@@ -25,6 +25,7 @@
 #include "pecan_contactlist_priv.h"
 #include "pecan_contact_priv.h"
 #include "pecan_log.h"
+#include "pecan_locale.h"
 
 /** @todo this is disabled for testing only */
 #ifdef HAVE_LIBPURPLE
@@ -35,8 +36,6 @@
 
 #include <string.h>
 
-#include "msn_intl.h"
-
 #define MSN_NULL_GROUP_NAME "Non-Grouped"
 
 #ifdef HAVE_LIBPURPLE
@@ -264,8 +263,7 @@ msn_got_add_contact (MsnSession *session,
 
         gc = purple_account_get_connection (account);
 
-        pecan_info ("rever list add: [%s]",
-                  passport);
+        pecan_info ("reverse list add: [%s]", passport);
 
         /** @todo display a non-intrusive message */
 #if 0
@@ -335,8 +333,7 @@ msn_got_rem_contact (MsnSession *session,
     {
         PurpleConversation *convo;
 
-        pecan_info ("rever list rem: [%s]",
-                  passport);
+        pecan_info ("reverse list rem: [%s]", passport);
 
         convo = purple_find_conversation_with_account (PURPLE_CONV_TYPE_IM, passport, account);
         if (convo)
@@ -462,13 +459,6 @@ pecan_contactlist_new (MsnSession *session)
     contactlist->group_guids = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
     contactlist->null_group = pecan_group_new (contactlist, MSN_NULL_GROUP_NAME, NULL);
 
-    contactlist->buddy_icon_requests = g_queue_new ();
-
-    /* buddy_icon_window is the number of allowed simultaneous buddy icon requests.
-     * XXX With smarter rate limiting code, we could allow more at once... 5 was the limit set when
-     * we weren't retrieiving any more than 5 per MSN session. */
-    contactlist->buddy_icon_window = 1;
-
     return contactlist;
 }
 
@@ -480,13 +470,6 @@ pecan_contactlist_destroy (PecanContactList *contactlist)
     g_hash_table_destroy (contactlist->group_guids);
     g_hash_table_destroy (contactlist->group_names);
 
-    g_queue_free (contactlist->buddy_icon_requests);
-
-#ifdef HAVE_LIBPURPLE
-    if (contactlist->buddy_icon_request_timer)
-        purple_timeout_remove (contactlist->buddy_icon_request_timer);
-#endif /* HAVE_LIBPURPLE */
-
     g_free (contactlist);
 }
 
