@@ -44,10 +44,6 @@ struct PecanContactList
     GHashTable *group_names;
     GHashTable *group_guids;
     PecanGroup *null_group;
-
-    GQueue *buddy_icon_requests;
-    gint buddy_icon_window;
-    guint buddy_icon_request_timer;
 };
 
 #endif /* PECAN_CONTACTLIST_PRIV_H */
