@@ -61,8 +61,6 @@ static void send_ok(MsnSlpCall *slpcall, const char *branch,
 static void send_decline(MsnSlpCall *slpcall, const char *branch,
 						 const char *type, const char *content);
 
-void msn_request_user_display(PecanContact *user);
-
 /**************************************************************************
  * Util
  **************************************************************************/
@@ -268,6 +266,8 @@ static void
 got_sessionreq(MsnSlpCall *slpcall, const char *branch,
 			   const char *euf_guid, const char *context)
 {
+	pecan_debug ("euf_guid=[%s]", euf_guid);
+
 	if (!strcmp(euf_guid, "A4268EEC-FEC5-49E5-95C3-F126696BDBF6"))
 	{
 		/* Emoticon or UserDisplay */
@@ -304,6 +304,7 @@ got_sessionreq(MsnSlpCall *slpcall, const char *branch,
 			g_return_if_reached();
 		}
 
+		/* image is owned by a local object, not obj */
 		image = msn_object_get_image (obj);
 		if (!image)
 		{
@@ -314,6 +315,13 @@ got_sessionreq(MsnSlpCall *slpcall, const char *branch,
 
 		msn_object_destroy(obj);
 
+		{
+			gchar *tmp;
+			tmp = msn_object_to_string (obj);
+			pecan_info ("object requested: %s", tmp);
+			g_free (tmp);
+		}
+
 		slpsession = msn_slplink_find_slp_session(slplink,
 												  slpcall->session_id);
 
@@ -429,6 +437,8 @@ got_invite(MsnSlpCall *slpcall,
 
 	slplink = slpcall->slplink;
 
+        pecan_log ("type=%s", type);
+
 	if (!strcmp(type, "application/x-msnmsgr-sessionreqbody"))
 	{
 		char *euf_guid, *context;
@@ -552,10 +562,13 @@ got_ok(MsnSlpCall *slpcall,
 	g_return_if_fail(slpcall != NULL);
 	g_return_if_fail(type    != NULL);
 
+        pecan_log ("type=%s", type);
+
 	if (!strcmp(type, "application/x-msnmsgr-sessionreqbody"))
 	{
 #ifdef MSN_DIRECTCONN
-		if (slpcall->type == MSN_SLPCALL_DC)
+		if (slpcall->slplink->session->use_directconn &&
+                    slpcall->type == MSN_SLPCALL_DC)
 		{
 			/* First let's try a DirectConnection. */
 
@@ -658,7 +671,7 @@ msn_slp_sip_recv(MsnSlpLink *slplink, const char *body)
 
 	if (body == NULL)
 	{
-		pecan_info ("received bogus message");
+		pecan_warning ("received bogus message");
 		return NULL;
 	}
 
@@ -817,9 +830,8 @@ got_emoticon(MsnSlpCall *slpcall,
 		purple_conv_custom_smiley_write(conv, slpcall->data_info, data, size);
 		purple_conv_custom_smiley_close(conv, slpcall->data_info);
 	}
-#ifdef PECAN_DEBUG_UD
-	pecan_info ("got smiley: %s", slpcall->data_info);
-#endif
+
+	pecan_debug ("got smiley: %s", slpcall->data_info);
 }
 
 void
@@ -856,7 +868,12 @@ msn_emoticon_msg(MsnCmdProc *cmdproc, MsnMessage *msg)
 		}
 
 		smile = tokens[tok];
-		obj = msn_object_new_from_string(purple_url_decode(tokens[tok + 1]));
+		{
+			gchar *tmp;
+			tmp = pecan_url_decode (tokens[tok + 1]);
+			obj = msn_object_new_from_string(tmp);
+			g_free(tmp);
+		}
 
 		if (obj == NULL)
 			break;
@@ -890,255 +907,3 @@ msn_emoticon_msg(MsnCmdProc *cmdproc, MsnMessage *msg)
 	}
 	g_strfreev(tokens);
 }
-
-static gboolean
-buddy_icon_cached(PurpleConnection *gc, MsnObject *obj)
-{
-	PurpleAccount *account;
-	PurpleBuddy *buddy;
-	const char *old;
-	const char *new;
-
-	g_return_val_if_fail(obj != NULL, FALSE);
-
-	account = purple_connection_get_account(gc);
-
-	buddy = purple_find_buddy(account, msn_object_get_creator(obj));
-	if (buddy == NULL)
-		return FALSE;
-
-	old = purple_buddy_icons_get_checksum_for_user(buddy);
-	new = msn_object_get_sha1(obj);
-
-	if (new == NULL)
-		return FALSE;
-
-	/* If the old and new checksums are the same, and the file actually exists,
-	 * then return TRUE */
-	if (old != NULL && !strcmp(old, new))
-		return TRUE;
-
-	return FALSE;
-}
-
-static void
-msn_release_buddy_icon_request(PecanContactList *contactlist)
-{
-	PecanContact *user;
-
-	g_return_if_fail(contactlist != NULL);
-
-#ifdef PECAN_DEBUG_UD
-	pecan_info ("releasing buddy icon request");
-#endif
-
-	if (contactlist->buddy_icon_window > 0)
-	{
-		GQueue *queue;
-		PurpleAccount *account;
-		const char *username;
-
-		queue = contactlist->buddy_icon_requests;
-
-		if (g_queue_is_empty(contactlist->buddy_icon_requests))
-			return;
-
-		user = g_queue_pop_head(queue);
-
-		account  = contactlist->session->account;
-		username = user->passport;
-
-		contactlist->buddy_icon_window--;
-		msn_request_user_display(user);
-
-#ifdef PECAN_DEBUG_UD
-                pecan_info ("msn_release_buddy_icon_request(): buddy_icon_window-- yields =%d",
-                          contactlist->buddy_icon_window);
-#endif
-	}
-}
-
-/*
- * Called on a timeout from end_user_display(). Frees a buddy icon window slow and dequeues the next
- * buddy icon request if there is one.
- */
-static gboolean
-msn_release_buddy_icon_request_timeout(gpointer data)
-{
-	PecanContactList *contactlist = (PecanContactList *)data;
-
-	/* Free one window slot */
-	contactlist->buddy_icon_window++;
-
-	/* Clear the tag for our former request timer */
-	contactlist->buddy_icon_request_timer = 0;
-
-	msn_release_buddy_icon_request(contactlist);
-
-	return FALSE;
-}
-
-void
-msn_queue_buddy_icon_request(PecanContact *user)
-{
-	PurpleAccount *account;
-	MsnObject *obj;
-	GQueue *queue;
-
-	g_return_if_fail(user != NULL);
-
-	account = user->contactlist->session->account;
-
-	obj = pecan_contact_get_object(user);
-
-	if (obj == NULL)
-	{
-		purple_buddy_icons_set_for_user(account, user->passport, NULL, 0, NULL);
-		return;
-	}
-
-	if (!buddy_icon_cached(account->gc, obj))
-	{
-		PecanContactList *contactlist;
-
-		contactlist = user->contactlist;
-		queue = contactlist->buddy_icon_requests;
-
-#ifdef PECAN_DEBUG_UD
-                pecan_info ("queueing buddy icon request for %s (buddy_icon_window = %i)",
-                          user->passport, contactlist->buddy_icon_window);
-#endif
-
-		g_queue_push_tail(queue, user);
-
-		if (contactlist->buddy_icon_window > 0)
-			msn_release_buddy_icon_request(contactlist);
-	}
-}
-
-static void
-got_user_display(MsnSlpCall *slpcall,
-				 const guchar *data, gsize size)
-{
-	PecanContactList *contactlist;
-	const char *info;
-	PurpleAccount *account;
-
-	g_return_if_fail(slpcall != NULL);
-
-	info = slpcall->data_info;
-#ifdef PECAN_DEBUG_UD
-	pecan_info ("got User Display: %s", slpcall->slplink->remote_user);
-#endif
-
-	contactlist = slpcall->slplink->session->contactlist;
-	account = slpcall->slplink->session->account;
-
-	purple_buddy_icons_set_for_user(account, slpcall->slplink->remote_user,
-									g_memdup(data, size), size, info);
-
-#if 0
-	/* Free one window slot */
-	contactlist->buddy_icon_window++;
-
-        pecan_info ("got_user_display(): buddy_icon_window++ yields =%d",
-                  contactlist->buddy_icon_window);
-
-	msn_release_buddy_icon_request(contactlist);
-#endif
-}
-
-static void
-end_user_display(MsnSlpCall *slpcall, MsnSession *session)
-{
-	PecanContactList *contactlist;
-
-	g_return_if_fail(session != NULL);
-
-#ifdef PECAN_DEBUG_UD
-	pecan_info ("End User Display");
-#endif
-
-	contactlist = session->contactlist;
-
-	/* If the session is being destroyed we better stop doing anything. */
-	if (session->destroying)
-		return;
-
-	/* Delay before freeing a buddy icon window slot and requesting the next icon, if appropriate.
-	 * If we don't delay, we'll rapidly hit the MSN equivalent of AIM's rate limiting; the server will
-	 * send us an error 800 like so:
-	 *
-	 * C: NS 000: XFR 21 SB
-	 * S: NS 000: 800 21
-	 */
-	if (contactlist->buddy_icon_request_timer) {
-		/* Free the window slot used by this previous request */
-		contactlist->buddy_icon_window++;
-
-		/* Clear our pending timeout */
-		purple_timeout_remove(contactlist->buddy_icon_request_timer);
-	}
-
-	/* Wait BUDDY_ICON_DELAY ms before freeing our window slot and requesting the next icon. */
-	contactlist->buddy_icon_request_timer = purple_timeout_add(BUDDY_ICON_DELAY,
-															msn_release_buddy_icon_request_timeout, contactlist);
-}
-
-void
-msn_request_user_display(PecanContact *user)
-{
-	PurpleAccount *account;
-	MsnSession *session;
-	MsnSlpLink *slplink;
-	MsnObject *obj;
-	const char *info;
-
-	session = user->contactlist->session;
-	account = session->account;
-
-	slplink = msn_session_get_slplink(session, user->passport);
-
-	obj = pecan_contact_get_object(user);
-
-	info = msn_object_get_sha1(obj);
-
-	if (g_ascii_strcasecmp(user->passport,
-						   purple_account_get_username(account)))
-	{
-		msn_slplink_request_object(slplink, info, got_user_display,
-								   end_user_display, obj);
-	}
-	else
-	{
-		MsnObject *my_obj = NULL;
-		gconstpointer data = NULL;
-		size_t len = 0;
-
-#ifdef PECAN_DEBUG_UD
-		pecan_info ("requesting our own user display");
-#endif
-
-		my_obj = pecan_contact_get_object(msn_session_get_contact (session));
-
-		if (my_obj != NULL)
-		{
-                    PecanBuffer *image;
-                    image = msn_object_get_image (my_obj);
-                    data = image->data;
-                    len = image->len;
-		}
-
-		purple_buddy_icons_set_for_user(account, user->passport, g_memdup(data, len), len, info);
-
-		/* Free one window slot */
-		session->contactlist->buddy_icon_window++;
-
-#ifdef PECAN_DEBUG_UD
-                pecan_info ("msn_request_user_display(): buddy_icon_window++ yields =%d",
-                          session->contactlist->buddy_icon_window);
-#endif
-
-		msn_release_buddy_icon_request(session->contactlist);
-	}
-}
