@@ -36,6 +36,8 @@
 #include "session.h"
 
 #ifdef HAVE_LIBPURPLE
+#include "fix_purple.h"
+
 /* libpurple stuff. */
 #include "fix_purple_win32.h"
 #include <proxy.h>
@@ -99,23 +101,30 @@ read_cb (GIOChannel *source,
 
     pecan_debug ("conn=%p,source=%p", conn, source);
 
+    g_object_ref (conn);
+
     {
         GIOStatus status = G_IO_STATUS_NORMAL;
 
         status = pecan_node_read (conn, buf, MSN_BUF_LEN, &bytes_read, &conn->error);
 
         if (status == G_IO_STATUS_AGAIN)
+        {
+            g_object_unref (conn);
             return TRUE;
+        }
 
         if (conn->error)
         {
             pecan_node_error (conn);
+            g_object_unref (conn);
             return FALSE;
         }
 
         if (status != G_IO_STATUS_NORMAL)
         {
             pecan_warning ("not normal, status=%d", status);
+            g_object_unref (conn);
             return TRUE;
         }
     }
@@ -126,11 +135,18 @@ read_cb (GIOChannel *source,
 
         http_conn = PECAN_HTTP_SERVER (conn);
 
-        g_object_ref (http_conn);
-
         if (http_conn->cur)
         {
-            pecan_node_parse (http_conn->cur, buf, bytes_read);
+            /* make sure the server is not sending the same buffer again */
+            /** @todo find out why this happens */
+            if (!(http_conn->old_buffer &&
+                  strncmp (buf, http_conn->old_buffer, bytes_read) == 0))
+            {
+                pecan_node_parse (http_conn->cur, buf, bytes_read);
+
+                g_free (http_conn->old_buffer);
+                http_conn->old_buffer = g_strndup (buf, bytes_read);
+            }
         }
 
         http_conn->waiting_response = FALSE;
@@ -140,13 +156,13 @@ read_cb (GIOChannel *source,
         if (conn->error)
         {
             pecan_node_error (conn);
-            g_object_unref (http_conn);
+            g_object_unref (conn);
             return FALSE;
         }
-
-        g_object_unref (http_conn);
     }
 
+    g_object_unref (conn);
+
     pecan_log ("end");
 
     return TRUE;
@@ -188,6 +204,7 @@ http_poll (gpointer data)
     GIOStatus status = G_IO_STATUS_NORMAL;
     GError *tmp_error = NULL;
     gsize bytes_written = 0;
+    static guint count = 0;
 
     gchar *header;
     gchar *params;
@@ -205,7 +222,11 @@ http_poll (gpointer data)
 
     g_return_val_if_fail (http_conn->cur, FALSE);
 
-    if (http_conn->waiting_response)
+    count++;
+
+    /* Don't poll if we already sent something, unless we have been waiting for
+     * too long; a disconnection might have happened. */
+    if (http_conn->waiting_response && count < 10)
     {
         /* There's no need to poll if we're already waiting for a response */
         pecan_debug ("waiting for response");
@@ -254,7 +275,11 @@ http_poll (gpointer data)
 
     if (status != G_IO_STATUS_NORMAL)
     {
+        PecanNodeClass *class;
         pecan_error ("not normal: status=%d", status);
+        class = g_type_class_peek (PECAN_NODE_TYPE);
+        g_signal_emit (G_OBJECT (conn), class->error_sig, 0, conn);
+        return FALSE;
     }
 
     return TRUE;
@@ -279,13 +304,14 @@ connect_cb (gpointer data,
     {
         GIOChannel *channel;
 
+        pecan_stream_free (conn->stream);
         conn->stream = pecan_stream_new (source);
         channel = conn->stream->channel;
 
         g_io_channel_set_encoding (channel, NULL, NULL);
         g_io_channel_set_line_term (channel, "\r\n", 2);
 
-        http_conn->timeout_id = g_timeout_add (2 * 1000, http_poll, http_conn);
+        http_conn->timeout_id = g_timeout_add_seconds (2, http_poll, http_conn);
         conn->read_watch = g_io_add_watch (channel, G_IO_IN, read_cb, conn);
 
         {
@@ -298,6 +324,10 @@ connect_cb (gpointer data,
     {
         PecanNodeClass *class;
         class = g_type_class_peek (PECAN_NODE_TYPE);
+
+        conn->error = g_error_new_literal (PECAN_NODE_ERROR, PECAN_NODE_ERROR_OPEN,
+                                           error_message ? error_message : "Unable to connect");
+
         g_signal_emit (G_OBJECT (conn), class->error_sig, 0, conn);
     }
 
@@ -317,6 +347,14 @@ connect_impl (PecanNode *conn,
 
     if (!conn->stream)
     {
+        /* close a pending connection */
+        /* this can happen when reconecting before receiving the connection
+         * callback. */
+        if (conn->connect_data)
+        {
+            purple_proxy_connect_cancel (conn->connect_data);
+        }
+
         if (conn->prev->type == PECAN_NODE_NS)
             hostname = http_conn->gateway;
         port = 80;
@@ -371,6 +409,11 @@ close_impl (PecanNode *conn)
         }
     }
 
+#if 0
+    /** @todo check that there aren't recursive frees. */
+    g_hash_table_remove_all (http_conn->childs);
+#endif
+
     PECAN_NODE_CLASS (parent_class)->close (conn);
 
     pecan_log ("end");
@@ -425,7 +468,9 @@ read_impl (PecanNode *conn,
 
             tokens = g_strsplit (str, " ", 3);
 
-            if (!(tokens[0] && (strcmp (tokens[0], "HTTP/1.1") == 0)))
+            if (!(tokens[0] &&
+                  ((strcmp (tokens[0], "HTTP/1.1") == 0) ||
+                   (strcmp (tokens[0], "HTTP/1.0") == 0))))
             {
                 pecan_debug ("error reading: parse error");
                 tmp_error = g_error_new_literal (PECAN_NODE_ERROR, PECAN_NODE_ERROR_OPEN,
@@ -526,6 +571,7 @@ read_impl (PecanNode *conn,
 
                         if (strcmp (token, "SessionID") == 0)
                         {
+                            g_free (http_conn->last_session_id);
                             http_conn->last_session_id = g_strdup (tokens_c[1]);
                         }
                         else if (strcmp (token, "GW-IP") == 0)
@@ -565,8 +611,8 @@ read_impl (PecanNode *conn,
 		session_id = g_strndup (http_conn->last_session_id, t - http_conn->last_session_id);
 
                 child = g_hash_table_lookup (http_conn->childs, session_id);
-                pecan_info ("child=%p", child);
-                pecan_info ("sesison_id=[%s]", session_id);
+                pecan_log ("child=%p", child);
+                pecan_log ("sesison_id=[%s]", session_id);
 
                 if (http_conn->session && (strcmp (http_conn->session, "close") == 0))
                 {
@@ -607,10 +653,11 @@ read_impl (PecanNode *conn,
 
                     if (child)
                     {
+                        g_free (child->foo_data);
                         child->foo_data = g_strdup (http_conn->last_session_id);
                     }
 
-                    pecan_info ("session=%s", http_conn->session);
+                    pecan_debug ("session=%s", http_conn->session);
                 }
 
                 g_free (session_id);
@@ -645,6 +692,15 @@ leave:
     return status;
 }
 
+static inline void
+reset_timer (PecanHttpServer *http_conn)
+{
+    if (http_conn->timeout_id)
+        g_source_remove (http_conn->timeout_id);
+
+    http_conn->timeout_id = g_timeout_add_seconds (2, http_poll, http_conn);
+}
+
 static GIOStatus
 foo_write (PecanNode *conn,
            PecanNode *prev,
@@ -665,6 +721,8 @@ foo_write (PecanNode *conn,
     {
         gchar *params;
         gchar *header;
+        gchar *body = NULL;
+        gsize body_len;
         gchar *auth = NULL;
         gchar *session_id;
 
@@ -672,46 +730,73 @@ foo_write (PecanNode *conn,
 
         if (session_id)
         {
-            params = pecan_strdup_printf ("SessionID=%s",
-                                          session_id);
+            params = g_strdup_printf ("SessionID=%s",
+                                      session_id);
         }
         else
         {
-            params = pecan_strdup_printf ("Action=open&Server=%s&IP=%s",
-                                          prev->type == PECAN_NODE_NS ? "NS" : "SB",
-                                          prev->hostname);
+            params = g_strdup_printf ("Action=open&Server=%s&IP=%s",
+                                      prev->type == PECAN_NODE_NS ? "NS" : "SB",
+                                      prev->hostname);
         }
 
-        header = pecan_strdup_printf ("POST http://%s/gateway/gateway.dll?%s HTTP/1.1\r\n"
-                                      "Accept: */*\r\n"
-                                      "Accept-Language: en-us\r\n"
-                                      "User-Agent: MSMSGS\r\n"
-                                      "Host: %s\r\n"
-                                      "Proxy-Connection: Keep-Alive\r\n"
-                                      "%s" /* Proxy auth */
-                                      "Connection: Keep-Alive\r\n"
-                                      "Pragma: no-cache\r\n"
-                                      "Content-Type: application/x-msn-messenger\r\n"
-                                      "Content-Length: %d\r\n\r\n",
-                                      http_conn->gateway,
-                                      params,
-                                      http_conn->gateway,
-                                      auth ? auth : "",
-                                      count);
+        /** @todo investigate why this returns NULL sometimes. */
+        header = g_strdup_printf ("POST http://%s/gateway/gateway.dll?%s HTTP/1.1\r\n"
+                                  "Accept: */*\r\n"
+                                  "Accept-Language: en-us\r\n"
+                                  "User-Agent: MSMSGS\r\n"
+                                  "Host: %s\r\n"
+                                  "Proxy-Connection: Keep-Alive\r\n"
+                                  "%s" /* Proxy auth */
+                                  "Connection: Keep-Alive\r\n"
+                                  "Pragma: no-cache\r\n"
+                                  "Content-Type: application/x-msn-messenger\r\n"
+                                  "Content-Length: %d\r\n\r\n",
+                                  http_conn->gateway,
+                                  params,
+                                  http_conn->gateway,
+                                  auth ? auth : "",
+                                  count);
+
+        g_free (params);
 
 #ifdef PECAN_DEBUG_HTTP
         pecan_debug ("header=[%s]", header);
 #endif
 
-        g_free (params);
+        /** @todo this is inefficient */
+        if (header)
+        {
+            gsize header_len;
+            header_len = strlen (header);
+            body_len = header_len + count;
+            body = g_malloc (body_len);
+            memcpy (body, header, header_len);
+            memcpy (body + header_len, buf, count);
+            g_free (header);
+        }
 
-        status = pecan_stream_write_full (conn->stream, header, strlen (header), &bytes_written, &tmp_error);
+        if (body)
+        {
+            status = pecan_stream_write_full (conn->stream, body, body_len, &bytes_written, &tmp_error);
+
+            g_free (body);
+        }
+        else
+        {
+            /** @todo this shouldn't happen. */
+            pecan_error ("body is null!");
+            status = G_IO_STATUS_ERROR;
+        }
     }
 
+    /** @todo investigate why multiple g_io_channel_write has memory leaks. */
+#if 0
     if (status == G_IO_STATUS_NORMAL)
     {
         status = pecan_stream_write_full (conn->stream, buf, count, &bytes_written, &tmp_error);
     }
+#endif
 
     if (status == G_IO_STATUS_NORMAL)
         status = pecan_stream_flush (conn->stream, &tmp_error);
@@ -725,6 +810,7 @@ foo_write (PecanNode *conn,
         http_conn->cur = prev;
         g_object_ref (G_OBJECT (http_conn->cur));
 
+        reset_timer (http_conn);
     }
     else
     {
@@ -786,6 +872,11 @@ dispose (GObject *obj)
 {
     PecanHttpServer *http_conn = PECAN_HTTP_SERVER (obj);
 
+    pecan_log ("begin");
+
+    g_free (http_conn->old_buffer);
+    http_conn->old_buffer = NULL;
+
     g_free (http_conn->gateway);
     http_conn->gateway = NULL;
 
@@ -796,6 +887,8 @@ dispose (GObject *obj)
     http_conn->childs = NULL;
 
     G_OBJECT_CLASS (parent_class)->dispose (obj);
+
+    pecan_log ("end");
 }
 
 static void
