@@ -28,6 +28,7 @@
 #include "pecan_util.h"
 #include "pecan_status.h"
 #include "pecan_log.h"
+#include "pecan_locale.h"
 
 #include "switchboard.h"
 #include "notification.h"
@@ -39,7 +40,11 @@
 
 #include "ab/pecan_contact_priv.h"
 
+#include <string.h> /* For strcmp, strstr, strlen */
+
+#if defined(PECAN_CVR)
 #include "cvr/slplink.h"
+#endif /* defined(PECAN_CVR) */
 
 /* libpurple stuff. */
 #include "fix_purple_win32.h"
@@ -54,7 +59,10 @@
 #include <prpl.h>
 #include <util.h>
 #include <prefs.h>
-#include "internal.h"
+
+#define _Q(x) #x
+#define Q(x) _Q(x)
+#define PLUGIN_ID "prpl-" Q(PLUGIN_NAME)
 
 typedef struct
 {
@@ -63,25 +71,40 @@ typedef struct
 
 } MsnMobileData;
 
+/* exports */
+void msn_set_friendly_name (PurpleConnection *gc, const gchar *entry);
+
 static void msn_set_prp(PurpleConnection *gc, const char *type, const char *entry);
 
-/** @todo remove this crap */
-static const char *
-msn_normalize(const PurpleAccount *account, const char *str)
+static gboolean
+contact_is_account_quick (MsnSession *session,
+                          const gchar *passport)
 {
-	static char buf[BUF_LEN];
-	char *tmp;
+    gchar *normalized_passport;
 
-	g_return_val_if_fail(str != NULL, NULL);
+    normalized_passport = pecan_normalize (passport);
 
-	g_snprintf(buf, sizeof(buf), "%s%s", str,
-			   (strchr(str, '@') ? "" : "@hotmail.com"));
+    if (strcmp (msn_session_get_username (session), normalized_passport) == 0)
+    {
+        g_free (normalized_passport);
+        return TRUE;
+    }
 
-	tmp = g_utf8_strdown(buf, -1);
-	strncpy(buf, tmp, sizeof(buf));
-	g_free(tmp);
+    g_free (normalized_passport);
+    return FALSE;
+}
 
-	return buf;
+/** @todo remove this crap */
+static const gchar *
+normalize (const PurpleAccount *account,
+           const gchar *str)
+{
+    static gchar buf[0x800];
+    gchar *tmp;
+    tmp = pecan_normalize (str);
+    strncpy (buf, tmp, sizeof (buf));
+    g_free (tmp);
+    return buf;
 }
 
 static gboolean
@@ -138,15 +161,17 @@ msn_cmd_nudge(PurpleConversation *conv, const gchar *cmd, gchar **args, gchar **
 static void
 msn_act_id(PurpleConnection *gc, const char *entry)
 {
-	MsnCmdProc *cmdproc;
 	MsnSession *session;
-	PurpleAccount *account;
-
 	session = gc->proto_data;
-	cmdproc = session->notification->cmdproc;
-	account = purple_connection_get_account(gc);
+        msn_set_prp(gc, "MFN", entry ? entry : msn_session_get_username (session));
+}
 
-        msn_set_prp(gc, "MFN", entry ? entry : "");
+/** Adium needs this. */
+void
+msn_set_friendly_name (PurpleConnection *gc,
+                       const gchar *entry)
+{
+    msn_act_id (gc, entry);
 }
 
 static void
@@ -337,11 +362,9 @@ show_hotmail_inbox (PurplePluginAction *action)
     gc = (PurpleConnection *) action->context;
     session = gc->proto_data;
 
-    /** @todo what about people who don't want notifications but still check
-     * the email? */
-    if (!session->passport_info.mail_url)
+    if (session->passport_info.email_enabled != 1)
     {
-        purple_notify_error (gc, NULL,  _("This Hotmail account may not be active."), NULL);
+        purple_notify_error (gc, NULL,  _("This account's email is not enabled."), NULL);
         return;
     }
 
@@ -428,9 +451,10 @@ initiate_chat_cb(PurpleBlistNode *node, gpointer data)
 	swboard->flag = MSN_SB_FLAG_IM;
 
 	purple_conv_chat_add_user(PURPLE_CONV_CHAT(swboard->conv),
-							purple_account_get_username(buddy->account), NULL, PURPLE_CBFLAGS_NONE, TRUE);
+							msn_session_get_username(session), NULL, PURPLE_CBFLAGS_NONE, TRUE);
 }
 
+#if defined(PECAN_CVR)
 static void
 t_msn_xfer_init(PurpleXfer *xfer)
 {
@@ -474,20 +498,23 @@ msn_send_file(PurpleConnection *gc, const char *who, const char *file)
 static gboolean
 msn_can_receive_file(PurpleConnection *gc, const char *who)
 {
-	PurpleAccount *account;
-	char *normal;
+        MsnSession *session;
+	gchar *normal_who;
 	gboolean ret;
 
-	account = purple_connection_get_account(gc);
+        session = gc->proto_data;
 
-	normal = g_strdup(msn_normalize(account, purple_account_get_username(account)));
+	g_return_val_if_fail (session, FALSE);
 
-	ret = strcmp(normal, msn_normalize(account, who));
+	normal_who = pecan_normalize (who);
 
-	g_free(normal);
+	ret = strcmp(normal_who, msn_session_get_username (session));
+
+	g_free (normal_who);
 
 	return ret;
 }
+#endif /* defined(PECAN_CVR) */
 
 /**************************************************************************
  * Protocol Plugin ops
@@ -536,6 +563,9 @@ tooltip_text (PurpleBuddy *buddy,
     PurplePresence *presence;
     PurpleStatus *status;
 
+    if (!buddy)
+        return;
+
     presence = purple_buddy_get_presence (buddy);
     status = purple_presence_get_active_status (presence);
     user = buddy->proto_data;
@@ -617,7 +647,7 @@ static GList *
 msn_actions(PurplePlugin *plugin, gpointer context)
 {
 	PurpleConnection *gc = (PurpleConnection *)context;
-	PurpleAccount *account;
+	MsnSession *session;
 	const char *user;
 
 	GList *m = NULL;
@@ -651,8 +681,8 @@ msn_actions(PurplePlugin *plugin, gpointer context)
 			msn_show_set_mobile_pages);
 	m = g_list_append(m, act);
 
-	account = purple_connection_get_account(gc);
-	user = msn_normalize(account, purple_account_get_username(account));
+        session = gc->proto_data;
+	user = msn_session_get_username(session);
 
 	if ((strstr(user, "@hotmail.") != NULL) ||
 		(strstr(user, "@msn.com") != NULL))
@@ -683,22 +713,25 @@ blist_node_menu (PurpleBlistNode *node)
 
             user = buddy->proto_data;
 
-            if (user && user->mobile)
+            if (user)
             {
-                /** @todo why is there a special way to do this? */
-                act = purple_menu_action_new (_("Send to Mobile"),
-                                              PURPLE_CALLBACK (show_send_to_mobile_cb),
-                                              NULL, NULL);
-                m = g_list_append (m, act);
-            }
-        }
+                if (user->mobile)
+                {
+                    /** @todo why is there a special way to do this? */
+                    act = purple_menu_action_new (_("Send to Mobile"),
+                                                  PURPLE_CALLBACK (show_send_to_mobile_cb),
+                                                  NULL, NULL);
+                    m = g_list_append (m, act);
+                }
 
-        if (g_ascii_strcasecmp (buddy->name, purple_account_get_username (buddy->account)) != 0)
-        {
-            act = purple_menu_action_new (_("Initiate _Chat"),
-                                          PURPLE_CALLBACK (initiate_chat_cb),
-                                          NULL, NULL);
-            m = g_list_append(m, act);
+                if (!pecan_contact_is_account (user))
+                {
+                    act = purple_menu_action_new (_("Initiate _Chat"),
+                                                  PURPLE_CALLBACK (initiate_chat_cb),
+                                                  NULL, NULL);
+                    m = g_list_append(m, act);
+                }
+            }
         }
 
         return m;
@@ -765,28 +798,23 @@ send_im (PurpleConnection *gc,
          const gchar *message,
          PurpleMessageFlags flags)
 {
-    PurpleAccount *account;
+    MsnSession *session;
     gchar *msgformat;
     gchar *msgtext;
 
-    account = purple_connection_get_account (gc);
+    session = gc->proto_data;
 
-    /** @todo use internal status instead */
+    /* Send to mobile when contact is offline. */
     {
-        PurpleBuddy *buddy;
-        buddy = purple_find_buddy (account, who);
-        if (buddy)
+        PecanContact *contact;
+        contact = pecan_contactlist_find_contact (session->contactlist, who);
+        if (contact && !contact->status && contact->mobile)
         {
-            PurplePresence *p;
-            p = purple_buddy_get_presence (buddy);
-            if (purple_presence_is_status_primitive_active (p, PURPLE_STATUS_MOBILE))
-            {
-                gchar *text;
-                text = purple_markup_strip_html (message);
-                send_to_mobile (gc, who, text);
-                g_free (text);
-                return 1;
-            }
+            gchar *text;
+            text = purple_markup_strip_html (message);
+            send_to_mobile (gc, who, text);
+            g_free (text);
+            return 1;
         }
     }
 
@@ -798,7 +826,7 @@ send_im (PurpleConnection *gc,
         g_free (msgformat);
         g_free (msgtext);
 
-        return -E2BIG;
+        return -7; /* E2BIG */
     }
 
     {
@@ -810,14 +838,12 @@ send_im (PurpleConnection *gc,
         g_free (msgtext);
 
         /* a message to ourselves? */
-        if (g_ascii_strcasecmp (who, purple_account_get_username (account)) == 0)
+        if (contact_is_account_quick (session, who))
             return -1;
 
         {
-            MsnSession *session;
             MsnSwitchBoard *swboard;
 
-            session = gc->proto_data;
             swboard = msn_session_get_swboard (session, who, MSN_SB_FLAG_IM);
 
             msn_switchboard_send_msg (swboard, msg, TRUE);
@@ -834,18 +860,16 @@ send_typing (PurpleConnection *gc,
              const gchar *who,
              PurpleTypingState state)
 {
-    PurpleAccount *account;
     MsnSession *session;
     MsnSwitchBoard *swboard;
 
-    account = purple_connection_get_account (gc);
     session = gc->proto_data;
 
     if (state != PURPLE_TYPING)
         return 0;
 
     /* a message to ourselves? */
-    if (g_ascii_strcasecmp (who, purple_account_get_username (account)) == 0)
+    if (contact_is_account_quick (session, who))
         return MSN_TYPING_SEND_TIMEOUT;
 
     swboard = msn_session_find_swboard (session, who);
@@ -861,7 +885,7 @@ send_typing (PurpleConnection *gc,
         msg = msn_message_new (MSN_MSG_TYPING);
         msn_message_set_content_type (msg, "text/x-msmsgscontrol");
         msn_message_set_flag (msg, 'U');
-        msn_message_set_attr (msg, "TypingUser", purple_account_get_username (account));
+        msn_message_set_attr (msg, "TypingUser", msn_session_get_username (session));
         msn_message_set_bin_data (msg, "\r\n", 2);
 
         msn_switchboard_send_msg (swboard, msg, FALSE);
@@ -970,6 +994,9 @@ alias_buddy (PurpleConnection *gc,
     cmdproc = session->notification->cmdproc;
     contact = pecan_contactlist_find_contact (session->contactlist, name);
 
+    if (!session->server_alias)
+        return;
+
     if (alias && strlen (alias))
         alias = purple_url_encode (alias);
     else
@@ -1238,7 +1265,7 @@ chat_send (PurpleConnection *gc,
     swboard = msn_session_find_swboard_with_id (session, id);
 
     if (!swboard)
-        return -EINVAL;
+        return -22; /* EINVAL */
 
     if (!swboard->ready)
         return 0;
@@ -1253,7 +1280,7 @@ chat_send (PurpleConnection *gc,
         g_free (msgformat);
         g_free (msgtext);
 
-        return -E2BIG;
+        return -7; /* E2BIG */
     }
 
     msg = msn_message_new_plain (msgtext);
@@ -1264,7 +1291,7 @@ chat_send (PurpleConnection *gc,
     g_free (msgformat);
     g_free (msgtext);
 
-    serv_got_chat_in (gc, id, purple_account_get_username (account), 0, message, time (NULL));
+    serv_got_chat_in (gc, id, msn_session_get_username (session), flags, message, time (NULL));
 
     return 0;
 }
@@ -1345,8 +1372,45 @@ get_info (PurpleConnection *gc,
           const char *name)
 {
     PurpleNotifyUserInfo *user_info;
+    PecanContact *user;
+    PurpleBuddy *buddy;
+
     user_info = purple_notify_user_info_new ();
     purple_notify_user_info_add_pair (user_info, _("Username"), name);
+
+    buddy = purple_find_buddy (purple_connection_get_account (gc), name);
+    user = (buddy ? buddy->proto_data : NULL);
+
+    if (user)
+    {
+        const gchar *friendly_name;
+        friendly_name = pecan_contact_get_friendly_name (user);
+        if (friendly_name && strcmp (friendly_name, name) != 0)
+            purple_notify_user_info_add_pair (user_info, _("Friendly Name"), friendly_name);
+    }
+
+    tooltip_text (buddy, user_info, /* full? */ TRUE);
+
+    if (user)
+    {
+        const gchar *home_phone;
+        const gchar *mobile_phone;
+        const gchar *work_phone;
+
+        home_phone = pecan_contact_get_home_phone (user);
+        mobile_phone = pecan_contact_get_mobile_phone (user);
+        work_phone = pecan_contact_get_work_phone (user);
+
+        if (home_phone)
+            purple_notify_user_info_add_pair (user_info, _("Home Phone"), home_phone);
+
+        if (mobile_phone)
+            purple_notify_user_info_add_pair (user_info, _("Mobile Phone"), mobile_phone);
+
+        if (work_phone)
+            purple_notify_user_info_add_pair (user_info, _("Work Phone"), work_phone);
+    }
+
     {
         gchar *tmp;
         tmp = pecan_strdup_printf ("<a href=\"%s%s\">%s%s</a>",
@@ -1354,6 +1418,7 @@ get_info (PurpleConnection *gc,
         purple_notify_user_info_add_pair (user_info, _("Profile URL"), tmp);
         g_free (tmp);
     }
+
     purple_notify_userinfo (gc, name, user_info, NULL, NULL);
     purple_notify_user_info_destroy (user_info);
 }
@@ -1430,7 +1495,7 @@ static PurplePluginProtocolInfo prpl_info =
     rename_group, /* rename_group */
     NULL, /* buddy_free */
     convo_closed, /* convo_closed */
-    msn_normalize, /* normalize */
+    normalize, /* normalize */
     set_buddy_icon, /* set_buddy_icon */
     remove_group, /* remove_group */
     NULL, /* get_cb_real_name */
@@ -1439,9 +1504,15 @@ static PurplePluginProtocolInfo prpl_info =
     NULL, /* roomlist_get_list */
     NULL, /* roomlist_cancel */
     NULL, /* roomlist_expand_category */
+#if defined(PECAN_CVR)
     msn_can_receive_file, /* can_receive_file */
     msn_send_file, /* send_file */
     msn_new_xfer, /* new_xfer */
+#else
+    NULL, /* can_receive_file */
+    NULL, /* send_file */
+    NULL, /* new_xfer */
+#endif /* defined(PECAN_CVR) */
     msn_offline_message, /* offline_message */
     NULL, /* whiteboard_prpl_ops */
     NULL, /* send_raw */
@@ -1449,7 +1520,9 @@ static PurplePluginProtocolInfo prpl_info =
     NULL, /* unregister_user */
     msn_send_attention, /* send_attention */
     msn_attention_types, /* attention_types */
-
+#ifdef LIBPURPLE_NEW_API
+    sizeof (PurplePluginProtocolInfo), /* struct_size */
+#endif
     /* padding */
     NULL
 };
@@ -1465,13 +1538,13 @@ static PurplePluginInfo info =
     NULL, /**< dependencies */
     PURPLE_PRIORITY_DEFAULT, /**< priority */
 
-    "prpl-msn-pecan", /**< id */
+    PLUGIN_ID, /**< id */
     "WLM", /**< name */
     VERSION, /**< version */
     N_("WLM Protocol Plugin"), /**< summary */
     N_("WLM Protocol Plugin"), /**< description */
     "Felipe Contreras <felipe.contreras@gmail.com>", /**< author */
-    PURPLE_WEBSITE, /**< homepage */
+    "http://code.google.com/p/msn-pecan/", /**< homepage */
 
     load, /**< load */
     unload, /**< unload */
@@ -1509,14 +1582,20 @@ init_plugin (PurplePlugin *plugin)
 
         option = purple_account_option_bool_new (_("Use server-side alias"), "server_alias", FALSE);
         prpl_info.protocol_options = g_list_append (prpl_info.protocol_options, option);
+
+        option = purple_account_option_bool_new (_("Use direct connections"), "use_directconn", FALSE);
+        prpl_info.protocol_options = g_list_append (prpl_info.protocol_options, option);
+
+        option = purple_account_option_bool_new (_("Use user displays"), "use_userdisplay", TRUE);
+        prpl_info.protocol_options = g_list_append (prpl_info.protocol_options, option);
     }
 
     purple_cmd_register ("nudge", "", PURPLE_CMD_P_PRPL,
                          PURPLE_CMD_FLAG_IM | PURPLE_CMD_FLAG_PRPL_ONLY,
-                         "prpl-msn-pecan", msn_cmd_nudge,
+                         PLUGIN_ID, msn_cmd_nudge,
                          _("nudge: nudge a user to get their attention"), NULL);
 
     purple_prefs_remove ("/plugins/prpl/msn");
 }
 
-PURPLE_INIT_PLUGIN (msn-pecan, init_plugin, info)
+PURPLE_INIT_PLUGIN (PLUGIN_NAME, init_plugin, info)
