@@ -22,6 +22,7 @@
 
 #include "notification.h"
 #include "pecan_log.h"
+#include "pecan_locale.h"
 #include "sync.h"
 #include "nexus.h"
 
@@ -42,13 +43,13 @@
 #include "io/pecan_node_priv.h"
 #include "io/pecan_cmd_server_priv.h"
 
+#if defined(PECAN_CVR)
 #include "cvr/slplink.h" /* for slplink_destroy */
+#endif /* defined(PECAN_CVR) */
 
-#include "error.h" /* for error_get_text */
+#include "pecan_error.h"
 #include "pecan_util.h" /* for parse_socket */
 
-#include "msn_intl.h"
-
 #include <glib/gstdio.h>
 
 #include <string.h>
@@ -67,18 +68,19 @@ typedef struct
 } MsnAddBuddy;
 
 static void
-open_cb (PecanNode *conn)
+open_cb (PecanNode *conn,
+         MsnNotification *notification)
 {
     MsnSession *session;
     PecanCmdServer *cmd_conn;
 
     g_return_if_fail (conn != NULL);
 
+    pecan_log ("begin");
+
     session = conn->session;
     cmd_conn = CMD_PECAN_NODE (conn);
 
-    pecan_log ("begin");
-
     if (session->login_step == PECAN_LOGIN_STEP_START)
         msn_session_set_login_step (session, PECAN_LOGIN_STEP_HANDSHAKE);
     else
@@ -114,6 +116,8 @@ close_cb (PecanNode *conn,
         }
     }
 
+    pecan_node_close (PECAN_NODE (notification->conn));
+    notification->closed = TRUE;
     msn_session_set_error (notification->session, MSN_ERROR_SERVCONN, tmp);
 
     g_free (tmp);
@@ -129,19 +133,13 @@ error_handler (MsnCmdProc *cmdproc,
                gint error)
 {
     MsnNotification *notification;
-    char *tmp;
+    gchar *reason;
 
     notification = cmdproc->data;
     g_return_if_fail (notification);
 
-    {
-        const gchar *reason;
-
-        reason = msn_error_get_text (error);
-
-        pecan_error ("connection error: (NS):reason=[%s]", reason);
-        tmp = pecan_strdup_printf (_("Error on notification server:\n%s"), reason);
-    }
+    reason = pecan_error_to_string (error);
+    pecan_error ("connection error: (NS):reason=[%s]", reason);
 
     switch (error)
     {
@@ -150,10 +148,15 @@ error_handler (MsnCmdProc *cmdproc,
             /* non-fatal */
             break;
         default:
-            msn_session_set_error (notification->session, MSN_ERROR_SERVCONN, tmp);
+            {
+                char *tmp;
+                tmp = pecan_strdup_printf (_("Error on notification server:\n%s"), reason);
+                msn_session_set_error (notification->session, MSN_ERROR_SERVCONN, tmp);
+                g_free (tmp);
+            }
     }
 
-    g_free (tmp);
+    g_free (reason);
 }
 
 MsnNotification *
@@ -188,20 +191,21 @@ msn_notification_new(MsnSession *session)
 
         if (session->http_method)
         {
-            PecanNode *foo;
-
             if (session->http_conn)
             {
-                foo = session->http_conn;
+                /* A single http connection shared by all nodes */
+                pecan_node_link (conn, session->http_conn);
             }
             else
             {
+                /* Each node has it's own http connection. */
+                PecanNode *foo;
+
                 foo = PECAN_NODE (pecan_http_server_new ("foo server"));
                 foo->session = session;
-                notification->http_conn = foo;
+                pecan_node_link (conn, foo);
+                g_object_unref (foo);
             }
-
-            pecan_node_link (conn, foo);
         }
 
         notification->open_handler = g_signal_connect (conn, "open", G_CALLBACK (open_cb), notification);
@@ -222,8 +226,6 @@ msn_notification_destroy(MsnNotification *notification)
     g_signal_handler_disconnect (notification->conn, notification->close_handler);
     g_signal_handler_disconnect (notification->conn, notification->error_handler);
 
-    if (notification->http_conn)
-        pecan_node_free (notification->http_conn);
     pecan_cmd_server_free (notification->conn);
 
     g_free(notification);
@@ -291,18 +293,22 @@ msn_got_login_params(MsnSession *session, const char *login_params)
 
     msn_session_set_login_step(session, PECAN_LOGIN_STEP_AUTH_END);
 
+    {
+        gchar **tokens;
+        tokens = g_strsplit (login_params, "&", 2);
+        session->passport_cookie.t = g_strdup (tokens[0] + 2);
+        session->passport_cookie.p = g_strdup (tokens[1] + 2);
+        g_strfreev (tokens);
+    }
+
     msn_cmdproc_send(cmdproc, "USR", "TWN S %s", login_params);
 }
 
 static void
 cvr_cmd(MsnCmdProc *cmdproc, MsnCommand *cmd)
 {
-    PurpleAccount *account;
-
-    account = cmdproc->session->account;
-
     msn_cmdproc_send(cmdproc, "USR", "TWN I %s",
-                     purple_account_get_username(account));
+                     msn_session_get_username(cmdproc->session));
 }
 
 static void
@@ -378,13 +384,11 @@ static void
 ver_cmd(MsnCmdProc *cmdproc, MsnCommand *cmd)
 {
     MsnSession *session;
-    PurpleAccount *account;
     gboolean protocol_supported = FALSE;
     const gchar *proto_str;
     guint i;
 
     session = cmdproc->session;
-    account = session->account;
 
     proto_str = "MSNP12";
 
@@ -406,7 +410,7 @@ ver_cmd(MsnCmdProc *cmdproc, MsnCommand *cmd)
 
     msn_cmdproc_send(cmdproc, "CVR",
                      "0x0409 winnt 5.1 i386 MSNMSGR 6.0.0602 MSMSGS %s",
-                     purple_account_get_username(account));
+                     msn_session_get_username(session));
 }
 
 /**************************************************************************
@@ -428,11 +432,11 @@ msn_notification_close(MsnNotification *notification)
 {
     g_return_if_fail(notification != NULL);
 
-    msn_cmdproc_send_quick(notification->cmdproc, "OUT", NULL, NULL);
-
-    if (notification->http_conn)
-        pecan_node_close (notification->http_conn);
-    pecan_node_close (PECAN_NODE (notification->conn));
+    if (!notification->closed)
+    {
+        msn_cmdproc_send_quick (notification->cmdproc, "OUT", NULL, NULL);
+        pecan_node_close (PECAN_NODE (notification->conn));
+    }
 }
 
 /**************************************************************************
@@ -531,7 +535,7 @@ adc_cmd(MsnCmdProc *cmdproc, MsnCommand *cmd)
     PecanContact *user = NULL;
     const gchar *list = NULL;
     const gchar *passport = NULL;
-    const gchar *friendly = NULL;
+    gchar *friendly = NULL;
     const gchar *user_guid = NULL;
     const gchar *group_guid = NULL;
     MsnListId list_id;
@@ -550,7 +554,7 @@ adc_cmd(MsnCmdProc *cmdproc, MsnCommand *cmd)
             passport = chopped_str;
         /* Check for Friendlyname. */
         else if (strncmp (cmd->params[i], "F=", 2) == 0)
-            friendly = purple_url_decode (chopped_str);
+            friendly = pecan_url_decode (chopped_str);
         /* Check for Contact GUID. */
         else if (strncmp (cmd->params[i], "C=", 2) == 0)
             user_guid = chopped_str;
@@ -571,13 +575,13 @@ adc_cmd(MsnCmdProc *cmdproc, MsnCommand *cmd)
     {
         user = pecan_contact_new (session->contactlist);
         pecan_contact_set_passport (user, passport);
-        pecan_contact_set_guid (user, user_guid);
     }
 
-    pecan_contact_set_guid (user, user_guid);
-
     list_id = msn_get_list_id(list);
 
+    if (list_id == MSN_LIST_FL)
+        pecan_contact_set_guid (user, user_guid);
+
     msn_got_add_contact(session, user, list_id, group_guid);
 
     /* There is a user that must me moved to this group */
@@ -593,6 +597,8 @@ adc_cmd(MsnCmdProc *cmdproc, MsnCommand *cmd)
     }
 
     pecan_contact_update(user);
+
+    g_free (friendly);
 }
 
 static void
@@ -627,7 +633,7 @@ adc_error(MsnCmdProc *cmdproc, MsnTransaction *trans, int error)
                                   purple_account_get_username(account),
                                   purple_account_get_protocol_name(account));
 
-    reason = msn_error_get_text(error);
+    reason = pecan_error_to_string (error);
 
     if (msg != NULL)
     {
@@ -653,13 +659,13 @@ adg_cmd(MsnCmdProc *cmdproc, MsnCommand *cmd)
 {
     MsnSession *session;
     const gchar *group_guid;
-    const char *group_name;
+    char *group_name;
 
     session = cmdproc->session;
 
     group_guid = cmd->params[2];
 
-    group_name = purple_url_decode(cmd->params[1]);
+    group_name = pecan_url_decode(cmd->params[1]);
 
     pecan_group_new(session->contactlist, group_name, group_guid);
 
@@ -680,6 +686,8 @@ adg_cmd(MsnCmdProc *cmdproc, MsnCommand *cmd)
         }
         g_free (data);
     }
+
+    g_free (group_name);
 }
 
 static void
@@ -691,19 +699,22 @@ qng_cmd(MsnCmdProc *cmdproc, MsnCommand *cmd)
 static void
 fln_cmd(MsnCmdProc *cmdproc, MsnCommand *cmd)
 {
-    MsnSlpLink *slplink;
     PecanContact *user;
 
     user = pecan_contactlist_find_contact(cmdproc->session->contactlist, cmd->params[0]);
-
-    user->status = "offline";
+    pecan_contact_set_state(user, NULL);
     pecan_contact_update(user);
 
-    slplink = msn_session_find_slplink(cmdproc->session, cmd->params[0]);
+#if defined(PECAN_CVR)
+    {
+        MsnSlpLink *slplink;
 
-    if (slplink != NULL)
-        msn_slplink_destroy(slplink);
+        slplink = msn_session_find_slplink(cmdproc->session, cmd->params[0]);
 
+        if (slplink != NULL)
+            msn_slplink_destroy(slplink);
+    }
+#endif /* defined(PECAN_CVR) */
 }
 
 static void
@@ -713,8 +724,11 @@ iln_cmd(MsnCmdProc *cmdproc, MsnCommand *cmd)
     PurpleAccount *account;
     PurpleConnection *gc;
     PecanContact *user;
+#if defined(PECAN_CVR)
     MsnObject *msnobj;
-    const char *state, *passport, *friendly;
+#endif /* defined(PECAN_CVR) */
+    const char *state, *passport;
+    gchar *friendly;
 
     session = cmdproc->session;
     account = session->account;
@@ -722,20 +736,37 @@ iln_cmd(MsnCmdProc *cmdproc, MsnCommand *cmd)
 
     state    = cmd->params[1];
     passport = cmd->params[2];
-    friendly = purple_url_decode(cmd->params[3]);
+    friendly = pecan_url_decode(cmd->params[3]);
 
     user = pecan_contactlist_find_contact(session->contactlist, passport);
 
+    pecan_contact_set_state(user, state);
     pecan_contact_set_friendly_name(user, friendly);
 
-    if (session->protocol_ver >= 9 && cmd->param_count == 6)
+    if (cmd->param_count >= 5)
     {
-        msnobj = msn_object_new_from_string(purple_url_decode(cmd->params[5]));
-        pecan_contact_set_object(user, msnobj);
+        gulong client_id;
+        client_id = atol (cmd->params[4]);
+        pecan_contact_set_client_id (user, client_id);
     }
 
-    pecan_contact_set_state(user, state);
+#if defined(PECAN_CVR)
+    if (session->use_userdisplay)
+    {
+        if (cmd->param_count == 6)
+        {
+            gchar *tmp;
+            tmp = pecan_url_decode (cmd->params[5]);
+            msnobj = msn_object_new_from_string (tmp);
+            pecan_contact_set_object(user, msnobj);
+            g_free (tmp);
+        }
+    }
+#endif /* defined(PECAN_CVR) */
+
     pecan_contact_update(user);
+
+    g_free (friendly);
 }
 
 static void
@@ -758,9 +789,12 @@ nln_cmd(MsnCmdProc *cmdproc, MsnCommand *cmd)
     PurpleAccount *account;
     PurpleConnection *gc;
     PecanContact *user;
+#if defined(PECAN_CVR)
     MsnObject *msnobj;
+#endif /* defined(PECAN_CVR) */
     int clientid;
-    const char *state, *passport, *friendly;
+    const char *state, *passport;
+    gchar *friendly;
 
     session = cmdproc->session;
     account = session->account;
@@ -768,7 +802,7 @@ nln_cmd(MsnCmdProc *cmdproc, MsnCommand *cmd)
 
     state    = cmd->params[0];
     passport = cmd->params[1];
-    friendly = purple_url_decode(cmd->params[2]);
+    friendly = pecan_url_decode(cmd->params[2]);
 
     user = pecan_contactlist_find_contact(session->contactlist, passport);
 
@@ -780,21 +814,37 @@ nln_cmd(MsnCmdProc *cmdproc, MsnCommand *cmd)
 
     pecan_contact_set_friendly_name(user, friendly);
 
-    if (cmd->param_count == 5)
+#if defined(PECAN_CVR)
+    if (session->use_userdisplay)
     {
-        msnobj = msn_object_new_from_string(purple_url_decode(cmd->params[4]));
-        pecan_contact_set_object(user, msnobj);
-    }
-    else
-    {
-        pecan_contact_set_object(user, NULL);
+        if (cmd->param_count == 5)
+        {
+            gchar *tmp;
+            tmp = pecan_url_decode(cmd->params[4]);
+            msnobj = msn_object_new_from_string(tmp);
+            pecan_contact_set_object(user, msnobj);
+            g_free (tmp);
+        }
+        else
+        {
+            pecan_contact_set_object(user, NULL);
+        }
     }
+#endif /* defined(PECAN_CVR) */
 
     clientid = atoi(cmd->params[3]);
     user->mobile = (clientid & MSN_CLIENT_CAP_MSNMOBILE);
 
     pecan_contact_set_state(user, state);
     pecan_contact_update(user);
+
+#if 0
+    /* store the friendly name on the server. */
+    if (!session->server_alias)
+        msn_cmdproc_send (cmdproc, "SBP", "%s %s %s", pecan_contact_get_guid (user), "MFN", friendly);
+#endif
+
+    g_free (friendly);
 }
 
 #if 0
@@ -888,7 +938,7 @@ sbp_cmd (MsnCmdProc *cmdproc,
     session = cmdproc->session;
     contact_guid = cmd->params[1];
     type = cmd->params[2];
-    value = purple_url_decode (cmd->params[3]);
+    value = cmd->params[3];
 
     contact = pecan_contactlist_find_contact_by_guid (session->contactlist, contact_guid);
 
@@ -896,14 +946,11 @@ sbp_cmd (MsnCmdProc *cmdproc,
     {
         if (strcmp (type, "MFN") == 0)
         {
+            gchar *tmp;
+            tmp = pecan_url_decode (value);
             if (session->server_alias)
-            {
-                pecan_contact_set_store_name (contact, value);
-            }
-            else
-            {
-                pecan_contact_set_friendly_name (contact, value);
-            }
+                pecan_contact_set_store_name (contact, tmp);
+            g_free (tmp);
         }
     }
 }
@@ -923,15 +970,18 @@ prp_cmd(MsnCmdProc *cmdproc, MsnCommand *cmd)
 
     if (cmd->param_count == 3)
     {
+        gchar *tmp;
         value = cmd->params[2];
+        tmp = pecan_url_decode (value);
         if (!strcmp(type, "PHH"))
-            pecan_contact_set_home_phone(user, purple_url_decode(value));
+            pecan_contact_set_home_phone(user, tmp);
         else if (!strcmp(type, "PHW"))
-            pecan_contact_set_work_phone(user, purple_url_decode(value));
+            pecan_contact_set_work_phone(user, tmp);
         else if (!strcmp(type, "PHM"))
-            pecan_contact_set_mobile_phone(user, purple_url_decode(value));
+            pecan_contact_set_mobile_phone(user, tmp);
         else if (!strcmp(type, "MFN"))
-            purple_connection_set_display_name(gc, purple_url_decode(value));
+            purple_connection_set_display_name(gc, tmp);
+        g_free (tmp);
     }
     else
     {
@@ -949,13 +999,15 @@ reg_cmd(MsnCmdProc *cmdproc, MsnCommand *cmd)
 {
     MsnSession *session;
     const gchar *group_guid;
-    const char *group_name;
+    char *group_name;
 
     session = cmdproc->session;
     group_guid = cmd->params[1];
-    group_name = purple_url_decode(cmd->params[2]);
+    group_name = pecan_url_decode(cmd->params[2]);
 
     pecan_contactlist_rename_group_id(session->contactlist, group_guid, group_name);
+
+    g_free (group_name);
 }
 
 static void
@@ -1055,12 +1107,19 @@ syn_cmd(MsnCmdProc *cmdproc, MsnCommand *cmd)
 
     total_users  = atoi(cmd->params[3]);
 
-    sync = msn_sync_new(session);
-    sync->total_users = total_users;
-    sync->old_cbs_table = cmdproc->cbs_table;
+    if (total_users > 0)
+    {
+	sync = msn_sync_new(session);
+	sync->total_users = total_users;
+	sync->old_cbs_table = cmdproc->cbs_table;
 
-    session->sync = sync;
-    cmdproc->cbs_table = sync->cbs_table;
+	session->sync = sync;
+	cmdproc->cbs_table = sync->cbs_table;
+    }
+    else
+    {
+	msn_session_finish_login (session);
+    }
 }
 
 static void
@@ -1186,6 +1245,7 @@ url_cmd (MsnCmdProc *cmdproc,
         return;
     }
 
+    if (purple_account_get_check_mail (session->account))
     {
         static gboolean is_initial = TRUE;
 
@@ -1286,8 +1346,10 @@ profile_msg(MsnCmdProc *cmdproc, MsnMessage *msg)
     session = cmdproc->session;
 
     if (strcmp(msg->remote_user, "Hotmail"))
-        /* This isn't an official message. */
+    {
+        pecan_warning ("unofficial message");
         return;
+    }
 
     if ((value = msn_message_get_attr(msg, "kv")) != NULL)
     {
@@ -1318,6 +1380,9 @@ profile_msg(MsnCmdProc *cmdproc, MsnMessage *msg)
 
     if ((value = msn_message_get_attr(msg, "LoginTime")) != NULL)
         session->passport_info.sl = atol(value);
+
+    if ((value = msn_message_get_attr(msg, "EmailEnabled")) != NULL)
+        session->passport_info.email_enabled = atol(value);
 }
 
 static void
@@ -1332,8 +1397,10 @@ initial_mdata_msg (MsnCmdProc *cmdproc,
     gc = session->account->gc;
 
     if (strcmp (msg->remote_user, "Hotmail"))
-        /* This isn't an official message. */
+    {
+        pecan_warning ("unofficial message");
         return;
+    }
 
     table = msn_message_get_hashtable_from_body (msg);
 
@@ -1366,66 +1433,62 @@ initial_mdata_msg (MsnCmdProc *cmdproc,
 
                 g_free (iu);
             }
+
+            do
+            {
+                start = g_strstr_len (start, len - (start - mdata), "<M>");
+
+                if (start)
+                {
+                    start += strlen ("<M>");
+                    end = g_strstr_len (start, len - (start - mdata), "</M>");
+
+                    if (end > start)
+                    {
+                        gchar *read_set;
+
+#if 0
+                        {
+                            gchar *field;
+                            gchar *tmp;
+                            tmp = pecan_get_xml_field ("N", start, end);
+                            field = purple_mime_decode_field (tmp);
+                            g_print ("field={%s}\n", field);
+                            g_free (field);
+                            g_free (tmp);
+                        }
+#endif
+
+                        read_set = pecan_get_xml_field ("RS", start, end);
+
+                        if (strcmp (read_set, "0") == 0)
+                        {
+                            gchar *passport;
+                            gchar *message_id;
+
+                            passport = pecan_get_xml_field ("E", start, end);
+
+                            message_id = pecan_get_xml_field ("I", start, end);
+
+                            pecan_oim_session_request (session->oim_session,
+                                                       passport,
+                                                       message_id);
+
+                            g_free (passport);
+                            g_free (message_id);
+                        }
+
+                        g_free (read_set);
+                        start = end + strlen ("</M>");
+                    }
+                }
+            } while (start);
         }
 
-        if (purple_account_get_check_mail (session->account))
+        if (purple_account_get_check_mail (session->account) &&
+            session->passport_info.email_enabled == 1)
         {
-            MsnTransaction *trans;
-            trans = msn_transaction_new (cmdproc, "URL", "%s", "INBOX");
-            msn_transaction_queue_cmd (trans, msg->cmd);
-
-            msn_cmdproc_send_trans (cmdproc, trans);
-        }
-    }
-
-    g_hash_table_destroy(table);
-}
-
-static void
-initial_email_msg(MsnCmdProc *cmdproc, MsnMessage *msg)
-{
-    MsnSession *session;
-    PurpleConnection *gc;
-    GHashTable *table;
-    const char *unread;
-
-    session = cmdproc->session;
-    gc = session->account->gc;
-
-    if (strcmp(msg->remote_user, "Hotmail"))
-        /* This isn't an official message. */
-        return;
-
-    if (!session->passport_info.mail_url)
-    {
-        MsnTransaction *trans;
-        trans = msn_transaction_new (cmdproc, "URL", "%s", "INBOX");
-        msn_transaction_queue_cmd (trans, msg->cmd);
-
-        msn_cmdproc_send_trans (cmdproc, trans);
-    }
-
-    if (!purple_account_get_check_mail(session->account))
-        return;
-
-    table = msn_message_get_hashtable_from_body(msg);
-
-    unread = g_hash_table_lookup(table, "Inbox-Unread");
-
-    if (unread != NULL)
-    {
-        int count = atoi(unread);
-
-        if (count > 0)
-        {
-            const char *passport;
-            const char *url;
-
-            passport = msn_session_get_username(session);
-            url = session->passport_info.mail_url;
-
-            purple_notify_emails(gc, atoi(unread), FALSE, NULL, NULL,
-                                 &passport, &url, NULL, NULL);
+            msn_cmdproc_send (cmdproc, "URL", "%s", "INBOX");
         }
     }
 
@@ -1443,23 +1506,21 @@ email_msg(MsnCmdProc *cmdproc, MsnMessage *msg)
     session = cmdproc->session;
     gc = session->account->gc;
 
-    if (strcmp(msg->remote_user, "Hotmail"))
-        /* This isn't an official message. */
+    if (!purple_account_get_check_mail (session->account))
         return;
 
-    if (!session->passport_info.mail_url)
+    if (strcmp(msg->remote_user, "Hotmail"))
     {
-        MsnTransaction *trans;
-        trans = msn_transaction_new (cmdproc, "URL", "%s", "INBOX");
-        msn_transaction_queue_cmd (trans, msg->cmd);
-
-        msn_cmdproc_send_trans (cmdproc, trans);
-
+        pecan_warning ("unofficial message");
         return;
     }
 
-    if (!purple_account_get_check_mail(session->account))
+    if (!session->passport_info.mail_url)
+    {
+        pecan_error ("no url");
         return;
+    }
+
 
     table = msn_message_get_hashtable_from_body(msg);
 
@@ -1493,8 +1554,10 @@ system_msg(MsnCmdProc *cmdproc, MsnMessage *msg)
     const char *type_s;
 
     if (strcmp(msg->remote_user, "Hotmail"))
-        /* This isn't an official message. */
+    {
+        pecan_warning ("unofficial message");
         return;
+    }
 
     table = msn_message_get_hashtable_from_body(msg);
 
@@ -1663,9 +1726,6 @@ msn_notification_init(void)
     msn_table_add_msg_type(cbs_table,
                            "text/x-msmsgsprofile",
                            profile_msg);
-    msn_table_add_msg_type(cbs_table,
-                           "text/x-msmsgsinitialemailnotification",
-                           initial_email_msg);
     msn_table_add_msg_type(cbs_table,
                            "text/x-msmsgsinitialmdatanotification",
                            initial_mdata_msg);
