@@ -20,9 +20,9 @@
 #define PECAN_CONFIG_H
 
 #define PACKAGE_NAME "msn-pecan"
-#define VERSION "0.0.13-git"
 #define PURPLE_PLUGINS
 
+#define MSN_BUF_LEN 8192
 #define PECAN_MAX_MESSAGE_LENGTH 1564
 
 #endif /* PECAN_CONFIG_H */
