@@ -40,6 +40,57 @@
 #define BUFSIZE 256
 #endif /* HAVE_LIBPURPLE */
 
+gchar *
+pecan_url_decode (const gchar *url)
+{
+    gchar *new;
+    const gchar *src;
+    gchar *dest;
+
+    /*
+       Thus, only alphanumerics, the special characters "$-_.+!*'(),", and
+       reserved characters used for their reserved purposes may be used
+       unencoded within a URL.
+       */
+
+    dest = new = g_malloc (strlen (url) + 1);
+    src = url;
+
+    while (*src != '\0')
+    {
+        if (*src == '%')
+        {
+            gint v1;
+            gint v2;
+
+            v1 = g_ascii_xdigit_value (src[1]);
+            v2 = g_ascii_xdigit_value (src[2]);
+
+            if ((v1 >= 0) && (v2 >= 0))
+            {
+                *dest = (v1 * 16) + v2;
+            }
+            else
+            {
+                /* Malformed */
+                g_free (new);
+                return NULL;
+            }
+            src += 3;
+        }
+        else
+        {
+            *dest = *src;
+            src++;
+        }
+        dest++;
+    }
+
+    *dest = '\0';
+
+    return new;
+}
+
 #ifdef HAVE_LIBPURPLE
 void
 msn_parse_format(const char *mime, char **pre_ret, char **post_ret)
@@ -558,6 +609,25 @@ msn_rand_guid()
                                rand() % 0xAAFF + 0x1111);
 }
 
+/** @todo remove this crap */
+gchar *
+pecan_normalize (const gchar *str)
+{
+    gchar *new;
+    gchar *tmp;
+
+    g_return_val_if_fail(str != NULL, NULL);
+
+    if (strchr (str, '@'))
+        return g_strdup (str);
+
+    tmp = g_utf8_strdown (str, -1);
+    new = g_strconcat (tmp, "@hotmail.com", NULL);
+    g_free (tmp);
+
+    return new;
+}
+
 static gboolean
 true_predicate (gpointer key,
                 gpointer value,
@@ -607,3 +677,36 @@ g_ascii_strcase_hash (gconstpointer v)
 
     return h;
 }
+
+/** @todo make this more efficient. */
+gchar *
+pecan_get_xml_field (const gchar *tag,
+                     const gchar *start,
+                     const gchar *end)
+{
+    const gchar *field_start;
+    const gchar *field_end;
+    gchar *field = NULL;
+    gchar *tag_start;
+    gchar *tag_end;
+
+    tag_start = g_strconcat ("<", tag, ">", NULL);
+    tag_end = g_strconcat ("</", tag, ">", NULL);
+
+    field_start = g_strstr_len (start, end - start, tag_start);
+    if (field_start)
+    {
+        field_start += strlen (tag_start);
+        field_end = g_strstr_len (field_start, field_start - end, tag_end);
+
+        if (field_end > field_start)
+        {
+            field = g_strndup (field_start, field_end - field_start);
+        }
+    }
+
+    g_free (tag_start);
+    g_free (tag_end);
+
+    return field;
+}
