@@ -24,7 +24,10 @@
 #include "session.h"
 #include "switchboard.h"
 #include "notification.h"
+
+#if defined(PECAN_CVR)
 #include "cvr/slplink.h"
+#endif /* defined(PECAN_CVR) */
 
 #include "session_private.h"
 
@@ -40,11 +43,9 @@
 
 #include "pecan_util.h"
 #include "pecan_log.h"
-
-#include "error.h"
+#include "pecan_locale.h"
 
 #include <string.h>
-#include "msn_intl.h"
 
 /* libpurple stuff. */
 #include "fix_purple_win32.h"
@@ -63,11 +64,11 @@ static void
 msn_switchboard_report_user(MsnSwitchBoard *swboard, PurpleMessageFlags flags, const char *msg);
 
 static void
-open_cb (PecanNode *conn)
+open_cb (PecanNode *conn,
+         MsnSwitchBoard *swboard)
 {
     MsnSession *session;
     PecanCmdServer *cmd_conn;
-    MsnSwitchBoard *swboard;
     MsnCmdProc *cmdproc;
 
     g_return_if_fail (conn != NULL);
@@ -76,11 +77,6 @@ open_cb (PecanNode *conn)
     cmd_conn = CMD_PECAN_NODE (conn);
     cmdproc = cmd_conn->cmdproc;
 
-    pecan_info ("foo");
-
-    swboard = cmdproc->data;
-    g_return_if_fail (swboard);
-
     {
         MsnTransaction *trans;
         PurpleAccount *account;
@@ -92,13 +88,13 @@ open_cb (PecanNode *conn)
             swboard->empty = FALSE;
 
             trans = msn_transaction_new (cmdproc, "ANS", "%s %s %s",
-                                         purple_account_get_username (account),
+                                         msn_session_get_username (session),
                                          swboard->auth_key, swboard->session_id);
         }
         else
         {
             trans = msn_transaction_new (cmdproc, "USR", "%s %s",
-                                         purple_account_get_username (account),
+                                         msn_session_get_username (session),
                                          swboard->auth_key);
         }
 
@@ -148,10 +144,9 @@ msn_switchboard_new(MsnSession *session)
     swboard->session = session;
 
     swboard->msg_queue = g_queue_new();
+    swboard->invites = g_queue_new();
     swboard->empty = TRUE;
 
-    session->switches = g_list_append(session->switches, swboard);
-
     {
         PecanNode *conn;
         swboard->conn = pecan_cmd_server_new ("switchboard server", PECAN_NODE_SB);
@@ -172,20 +167,21 @@ msn_switchboard_new(MsnSession *session)
 
         if (session->http_method)
         {
-            PecanNode *foo;
-
             if (session->http_conn)
             {
-                foo = session->http_conn;
+                /* A single http connection shared by all nodes */
+                pecan_node_link (conn, session->http_conn);
             }
             else
             {
+                /* Each node has it's own http connection. */
+                PecanNode *foo;
+
                 foo = PECAN_NODE (pecan_http_server_new ("foo server"));
                 foo->session = session;
-                swboard->http_conn = foo;
+                pecan_node_link (conn, foo);
+                g_object_unref (foo);
             }
-
-            pecan_node_link (conn, foo);
         }
 
         swboard->open_handler = g_signal_connect (conn, "open", G_CALLBACK (open_cb), swboard);
@@ -193,32 +189,56 @@ msn_switchboard_new(MsnSession *session)
         swboard->error_handler = g_signal_connect (conn, "error", G_CALLBACK (close_cb), swboard);
     }
 
+    session->switches = g_list_append(session->switches, swboard);
+
     return swboard;
 }
 
 void
 msn_switchboard_destroy(MsnSwitchBoard *swboard)
 {
-    MsnSession *session;
     MsnMessage *msg;
     GList *l;
 
     pecan_log ("begin");
 
-#ifdef PECAN_DEBUG_SB
-    pecan_log ("swboard=[%p]", swboard);
-#endif
+    pecan_log ("swboard=%p", swboard);
 
     g_return_if_fail(swboard);
 
     if (swboard->destroying)
+    {
+        pecan_log ("destroying");
+        pecan_log ("end");
         return;
+    }
 
     swboard->destroying = TRUE;
 
+    /* Make sure nobody find this swboard again. */
+    {
+        MsnSession *session;
+        session = swboard->session;
+        session->switches = g_list_remove (session->switches, swboard);
+    }
+
+    g_signal_handler_disconnect (swboard->conn, swboard->open_handler);
+    g_signal_handler_disconnect (swboard->conn, swboard->close_handler);
+    g_signal_handler_disconnect (swboard->conn, swboard->error_handler);
+
+#if defined(PECAN_CVR)
     /* If it linked us is because its looking for trouble */
     while (swboard->slplinks != NULL)
         msn_slplink_destroy(swboard->slplinks->data);
+#endif /* defined(PECAN_CVR) */
+
+    {
+        gchar *participant;
+        while ((participant = g_queue_pop_tail (swboard->invites)))
+            g_free (participant);
+    }
+
+    g_queue_free (swboard->invites);
 
     /* Destroy the message queue */
     while ((msg = g_queue_pop_head(swboard->msg_queue)) != NULL)
@@ -246,18 +266,14 @@ msn_switchboard_destroy(MsnSwitchBoard *swboard)
     for (l = swboard->users; l != NULL; l = l->next)
         g_free(l->data);
 
-    session = swboard->session;
-    session->switches = g_list_remove(session->switches, swboard);
-
     if (swboard->cmdproc)
         swboard->cmdproc->data = NULL;
 
-    g_signal_handler_disconnect (swboard->conn, swboard->open_handler);
-    g_signal_handler_disconnect (swboard->conn, swboard->close_handler);
-    g_signal_handler_disconnect (swboard->conn, swboard->error_handler);
+    /* make sure all the transactions are destroyed so no timeouts occur after
+     * the switchboard is destroyed; the node can still be refed by someone
+     * else. */
+    pecan_node_close (PECAN_NODE (swboard->conn));
 
-    if (swboard->http_conn)
-        pecan_node_free (swboard->http_conn);
     pecan_node_free (PECAN_NODE (swboard->conn));
 
     g_free(swboard);
@@ -340,13 +356,11 @@ send_clientcaps(MsnSwitchBoard *swboard)
 static void
 msn_switchboard_add_user(MsnSwitchBoard *swboard, const char *user)
 {
-    MsnCmdProc *cmdproc;
     PurpleAccount *account;
 
     g_return_if_fail(swboard);
 
-    cmdproc = swboard->cmdproc;
-    account = cmdproc->session->account;
+    account = swboard->session->account;
 
     swboard->users = g_list_prepend(swboard->users, g_strdup(user));
     swboard->current_users++;
@@ -388,7 +402,7 @@ msn_switchboard_add_user(MsnSwitchBoard *swboard, const char *user)
                 purple_conversation_destroy(swboard->conv);
 #endif
 
-            swboard->chat_id = cmdproc->session->conv_seq++;
+            swboard->chat_id = swboard->session->conv_seq++;
             swboard->flag |= MSN_SB_FLAG_IM;
             swboard->conv = serv_got_joined_chat(account->gc,
                                                  swboard->chat_id,
@@ -717,7 +731,7 @@ queue_msg(MsnSwitchBoard *swboard, MsnMessage *msg)
     g_return_if_fail(swboard);
     g_return_if_fail(msg     != NULL);
 
-    pecan_info ("appending message to queue");
+    pecan_debug ("appending message to queue");
 
     g_queue_push_tail(swboard->msg_queue, msg);
 
@@ -731,11 +745,11 @@ process_queue(MsnSwitchBoard *swboard)
 
     g_return_if_fail(swboard);
 
-    pecan_info ("processing queue");
+    pecan_debug ("processing queue");
 
     while ((msg = g_queue_pop_head(swboard->msg_queue)) != NULL)
     {
-        pecan_info ("sending message");
+        pecan_debug ("sending message");
         release_msg(swboard, msg);
         msn_message_unref(msg);
     }
@@ -964,8 +978,16 @@ usr_cmd(MsnCmdProc *cmdproc, MsnCommand *cmd)
     }
 #endif
 
+    {
+        gchar *participant;
+        while ((participant = g_queue_pop_head (swboard->invites)))
+        {
+            msn_cmdproc_send (cmdproc, "CAL", "%s", participant);
+            g_free (participant);
+        }
+    }
+
     swboard->ready = TRUE;
-    msn_cmdproc_process_queue(cmdproc);
 }
 
 /**************************************************************************
@@ -1035,8 +1057,7 @@ plain_msg(MsnCmdProc *cmdproc, MsnMessage *msg)
         /* If current_users is always ok as it should then there is no need to
          * check if this is a chat. */
         if (swboard->current_users <= 1)
-            pecan_info ("plain_msg: current_users=[%d]",
-                      swboard->current_users);
+            pecan_warning ("plain_msg: current_users=[%d]", swboard->current_users);
 
         serv_got_chat_in(gc, swboard->chat_id, passport, 0, body_final,
                          time(NULL));
@@ -1060,6 +1081,74 @@ plain_msg(MsnCmdProc *cmdproc, MsnMessage *msg)
     g_free(body_final);
 }
 
+#if defined(PECAN_CVR)
+void
+msn_handwritten_msg_show(MsnSwitchBoard *swboard, const char* msgid, const char* data, char* passport)
+{
+    guchar *guc;
+    size_t body_len;
+    PurpleAccount *account;
+    
+    account = swboard->session->account;
+    guc = purple_base64_decode(data, &body_len);
+    if (!guc || !body_len) 
+        return;
+    
+    /*Grab the convo for this sboard. If there isn't one and it's an IM 
+        then create it, otherwise the smileys won't work*/
+    if (swboard->conv == NULL)
+    {
+        if (swboard->current_users > 1) 
+            swboard->conv = purple_find_chat(account->gc, swboard->chat_id);
+        else
+        {
+            swboard->conv = purple_find_conversation_with_account(PURPLE_CONV_TYPE_IM,
+                                    passport, account);
+            if (swboard->conv == NULL)
+                swboard->conv = purple_conversation_new(PURPLE_CONV_TYPE_IM, account, passport);
+        }
+    }
+    swboard->flag |= MSN_SB_FLAG_IM;
+
+    /*Create the image as a custom smiley in the conversation with the GUID of the message that 
+        the image arrived in as the smiley text.*/
+    if (purple_conv_custom_smiley_add(swboard->conv, msgid, 0, "", 0)) {
+        purple_conv_custom_smiley_write(swboard->conv, msgid, guc, body_len);
+        purple_conv_custom_smiley_close(swboard->conv, msgid);
+    }
+    
+    /*And put the GUID into the convo window so that it will display...*/
+    if (swboard->current_users > 1 ||
+        ((swboard->conv != NULL) &&
+         purple_conversation_get_type(swboard->conv) == PURPLE_CONV_TYPE_CHAT))
+        serv_got_chat_in(account->gc, swboard->chat_id, passport, 0, msgid,
+                         time(NULL));
+    else
+        serv_got_im(account->gc, passport, msgid, 0, time(NULL));
+
+    g_free(guc);
+}
+
+/*only called from chats. Handwritten messages for IMs come as a SLP message*/
+static void
+msn_handwritten_msg(MsnCmdProc *cmdproc, MsnMessage *msg)
+{
+    char *passport, *bodydup;
+    const char *body, *msgid;
+    size_t body_len;
+    
+    passport = msg->remote_user;
+    msgid = msn_message_get_attr(msg, "Message-ID");
+
+    pecan_debug ("Displaying handwritten message from message %s", msgid);
+
+    body = msn_message_get_bin_data(msg, &body_len);
+    bodydup = g_strndup(body+7, body_len-7);
+    msn_handwritten_msg_show(cmdproc->data, msgid, bodydup, passport);
+    g_free(bodydup);
+}
+#endif /* defined(PECAN_CVR) */
+
 static void
 control_msg(MsnCmdProc *cmdproc, MsnMessage *msg)
 {
@@ -1190,7 +1279,7 @@ cal_error(MsnCmdProc *cmdproc, MsnTransaction *trans, int error)
 
     if (error == 215)
     {
-        pecan_info ("already in switchboard");
+        pecan_warning ("already in switchboard");
         return;
     }
     else if (error == 217)
@@ -1198,8 +1287,7 @@ cal_error(MsnCmdProc *cmdproc, MsnTransaction *trans, int error)
         reason = MSN_SB_ERROR_USER_OFFLINE;
     }
 
-    pecan_warning ("command=[%s],error=%i",
-                 trans->command, error);
+    pecan_warning ("command=[%s],error=%i",  trans->command, error);
 
     cal_error_helper(trans, reason);
 }
@@ -1214,6 +1302,13 @@ msn_switchboard_request_add_user(MsnSwitchBoard *swboard, const char *user)
 
     cmdproc = swboard->cmdproc;
 
+    if (!swboard->ready)
+    {
+        pecan_warning ("not ready yet");
+        g_queue_push_tail (swboard->invites, g_strdup (user));
+        return;
+    }
+
     trans = msn_transaction_new(cmdproc, "CAL", "%s", user);
     /* this doesn't do anything, but users seem to think that
      * 'Unhandled command' is some kind of error, so we don't report it */
@@ -1222,10 +1317,7 @@ msn_switchboard_request_add_user(MsnSwitchBoard *swboard, const char *user)
     msn_transaction_set_data(trans, swboard);
     msn_transaction_set_timeout_cb(trans, cal_timeout);
 
-    if (swboard->ready)
-        msn_cmdproc_send_trans(cmdproc, trans);
-    else
-        msn_cmdproc_queue_trans(cmdproc, trans);
+    msn_cmdproc_send_trans(cmdproc, trans);
 }
 
 /**************************************************************************
@@ -1273,9 +1365,8 @@ xfr_error(MsnCmdProc *cmdproc, MsnTransaction *trans, int error)
     swboard = trans->data;
     g_return_if_fail (swboard);
 
-    pecan_info ("error=%i,user=[%s],trans=%p,command=[%s],reason=%i",
-              error, swboard->im_user, trans,
-              trans->command, reason);
+    pecan_error ("error=%i,user=[%s],trans=%p,command=[%s],reason=%i",
+                 error, swboard->im_user, trans, trans->command, reason);
 
     swboard_error_helper(swboard, reason, swboard->im_user);
 }
@@ -1304,6 +1395,13 @@ msn_switchboard_close(MsnSwitchBoard *swboard)
 {
     g_return_if_fail(swboard);
 
+    /* Make sure nobody find this swboard again. */
+    {
+        MsnSession *session;
+        session = swboard->session;
+        session->switches = g_list_remove (session->switches, swboard);
+    }
+
     if (swboard->error != MSN_SB_ERROR_NONE)
     {
         msn_switchboard_destroy(swboard);
@@ -1312,8 +1410,15 @@ msn_switchboard_close(MsnSwitchBoard *swboard)
              !swboard->session->connected)
     {
         MsnCmdProc *cmdproc;
+        gboolean destroying;
+
         cmdproc = swboard->cmdproc;
+        destroying = swboard->destroying;
+
+        /* hack to inhibit destroying */
+        swboard->destroying = TRUE;
         msn_cmdproc_send_quick(cmdproc, "OUT", NULL, NULL);
+        swboard->destroying = destroying;
 
         msn_switchboard_destroy(swboard);
     }
@@ -1383,12 +1488,16 @@ msn_switchboard_init(void)
                            clientcaps_msg);
     msn_table_add_msg_type(cbs_table, "text/x-clientinfo",
                            clientcaps_msg);
+#if defined(PECAN_CVR)
     msn_table_add_msg_type(cbs_table, "application/x-msnmsgrp2p",
                            msn_p2p_msg);
     msn_table_add_msg_type(cbs_table, "text/x-mms-emoticon",
                            msn_emoticon_msg);
     msn_table_add_msg_type(cbs_table, "text/x-mms-animemoticon",
                            msn_emoticon_msg);
+    msn_table_add_msg_type(cbs_table, "image/gif",
+                           msn_handwritten_msg);
+#endif /* defined(PECAN_CVR) */
     msn_table_add_msg_type(cbs_table, "text/x-msnmsgr-datacast",
                            nudge_msg);
 #if 0
